#!/usr/bin/perl
#
# diablo-stats.pl v1.2  970421  Iain Lea  iain@ecrc.de
#
# Parse diablo logfile and generate stats about incoming & outgoing feeds.
#
# TODO
# - need a Kb/Mb per feed by disconnect/final lines to calc volume (1.09)
# - add colors for >25% (yellow)  >50% (orange)  >75% (red) to batches
#
# File format:
#
#  Incoming Feed
#  -------------
#  Apr 10 11:36:54 host diablo[12967]: Connection 21 from news.ecrc.de 206.229.87.10
#  Apr 10 11:37:06 host diablo[12967]: news.ecrc.de secs=12 ihave=9    chk=0    rec=7    rej=0    err=0    added=7    (0/sec)
#  Apr 10 11:37:06 host diablo[12967]: Disconnect 21 from news.ecrc.de 206.229.87.10 (0 elapsed)
#  ...
#
#  Outgoing Feed
#  -------------
#  Apr  9 12:20:35 host newslink[14596]: news.ecrc.de:news.ecrc.de.S00013 connect: 203 StreamOK. (streaming)
#  Apr  9 12:21:07 host newslink[14596]: news.ecrc.de:news.ecrc.de.S00013 mark secs=63   acc=0    dup=44   rej=955  tot=999  (951/min 0/15 mS) 439 <NEWT
#  Apr  9 12:21:18 host newslink[14596]: news.ecrc.de:news.ecrc.de.S00013 final secs=74   acc=0    dup=48   rej=1086 tot=1134 (919/min 0/18 mS) 439 <334A
#  ...
#
#  Statistics (will not be parsed)
#  -------------------------------
#  Apr  9 00:01:41 host diablo[7071]: DIABLO uptime=1:18 arts=71.000K tested=0 bytes=534.009M fed=594.171K
#  Apr  9 00:02:43 host diablo[7071]: DIABLO uptime=1:19 arts=72.000K tested=0 bytes=536.979M fed=602.620K
#  ...
#

require 'getopts.pl';

$LogFile = '/var/log/news/news.notice';
$QueueCmd = '/news/dbin/doutq';
$OutFile = '';
$HtmlDir = '';
$ImgDir = '/img';
$TextFormat = 0;
$Verbose = 0;
$Debug = 0;

chop ($DiabloHost = `hostname`);
chop ($GenYear = `date +%y`);
chop ($GenDate = `date +%y%m%d`);
chop ($GenTime = `date +%H%M`);
$TmpDir = "/tmp";
$MaxHeight = 250;
$Url = 'ftp://ftp.ecrc.de/pub/news/servers/diablo-stats/';
$ScriptName = 'diablo-stats';
$Version = 'v1.2';
$CreateIndexAndExit = 0;
$TotalRequests = 0;
@HalfHours = 0;
$BegDate = $EndDate = $GenDate;
$BegTime = $EndTime = "";
$TDL = "<td align=\"left\">";
$TDR = "<td align=\"right\">";
my (%MonthNumByName) = (
	'Jan', 1, 'Feb', 2, 'Mar', 3, 'Apr', 4,  'May', 5,  'Jun', 6,
	'Jul', 7, 'Aug', 8, 'Sep', 9, 'Oct', 10, 'Nov', 11, 'Dec', 12);
my (%MonthNameByNum) = (
	'01', 'Jan', '02', 'Feb', '03', 'Mar', '04', 'Apr', '05', 'May', '06', 'Jun',
	'07', 'Jul', '08', 'Aug', '09', 'Sep', '10', 'Oct', '11', 'Nov', '12', 'Dec');
# Colorize outgoing queue. Defaults are 25/40/55/70/85%
$QueColor1 = 'lightgreen';
$QueColor2 = 'lightblue';
$QueColor3 = 'lightyellow';
$QueColor4 = 'orange';
$QueColor5 = 'red';
$QuePercent1 = 25;
$QuePercent2 = 40;
$QuePercent3 = 55;
$QuePercent4 = 70;
$QuePercent5 = 85;

##############################################################################
# 
#

&ParseCmdLine ($0);

# &InitReqsPerHalfHour;

if ($CreateIndexAndExit) {
	&CreateHtmlIndex;
} else {
	&ParseLogFile;

	&ParseOutQueue;

	&GenerateOutputFile;

	&CreateHtmlIndex;
}

exit 0;

##############################################################################
# 
#

sub ParseCmdLine
{
	my ($ProgName) = @_;
	
	&Getopts('B:E:d:hl:o:s:tvVw:W:');

	if ($opt_h) { 
		print <<EOT
$ScriptName $Version  $Url

Create statistics for DIABLO news relay server in Ascii / HTML format.
Copyright 1997 Iain Lea (iain\@ecrc.de).    NOTE: Use at your own risk.

Usage: $ProgName [options]
       -h       help
       -d date  date to gather statistics in YYMMDD (default: $GenDate)
       -B date  begin time to start gathering statistics in YYMMDD
       -E date  end time to stop gathering statistics in YYMMDD
       -l file  access log file (default: $LogFile)
       -o file  output file (default: stdout)
       -s host  server hostname to use in report (default: $DiabloHost)
       -t       output format as text (default: HTML)
       -v       verbose
       -V       debug
       -w dir   create dir/index.html page to list daily statistics
       -W dir   create dir/index.html page to list daily statistics and exit

EOT
;
		exit 1;
	}
	$BegDate = $opt_B if (defined($opt_B));
	$EndDate = $opt_E if (defined($opt_E));
	$BegDate = $EndDate = $GenDate = $opt_d if (defined($opt_d));
	$LogFile = $opt_l if (defined($opt_l));
	$OutFile = $opt_o if (defined($opt_o));
	$DiabloHost = $opt_s if (defined($opt_s));
	$TextFormat++ if (defined($opt_t));
	$Verbose++ if (defined($opt_v));
	$Debug++ if (defined($opt_V));
	$HtmlDir = $opt_w if (defined($opt_w));

	if (defined($opt_W)) {
		$HtmlDir = $opt_W;
		$CreateIndexAndExit++;
	} else {
		die "Error: $LogFile - $!\n" if ($LogFile eq '');
	}
}


sub ParseLogFile
{
	print STDERR "Parsing $LogFile from $BegDate - $EndDate ...\n" if $Verbose;

	open (FILE, $LogFile) || die "Error: $LogFile - $!\n";

	while (<FILE>) 
	{
		chop;

		if (/(... .. ..:..:..)\s+(.*)\s+diablo\[(\d+)\]:\s+(.*)\s+secs=(\d+)\s+ihave=(\d+)\s+chk=(\d+)\s+rec=(\d+)\s+rej=(\d+)\s+err=(\d+)\s+added=(\d+)/) {

			$InDate = &GetYyMmDd ($1);

#			print STDERR "DATE  $GenDate  $InDate\n" if $Debug;

			if ($InDate >= $BegDate && $InDate <= $EndDate) {
#				$DiabloHost = $2;
#				$DiabloPid = $3;
				$InHost = $4;
				$InSecs = $5;
				$InIhave = $6;
				$InChk = $7;
				$InRec = $8;
				$InRej = $9;
				$InErr = $10;
				$InAdded = $11;

				$InHost =~ s/\s//g;
				$BegTime = $InDate if !$BegTime;
				$EndTime = $InDate;

				$Feeds{$InHost}->{InCons} += 1;
				$Feeds{$InHost}->{InSecs} += $InSecs;
				$Feeds{$InHost}->{InIhave} += $InIhave;
				$Feeds{$InHost}->{InChk} += $InChk;
				$Feeds{$InHost}->{InRec} += $InRec;
				$Feeds{$InHost}->{InRej} += $InRej;
				$Feeds{$InHost}->{InErr} += $InErr;
				$Feeds{$InHost}->{InAdded} += $InAdded;

				printf STDERR "IN  %-30s %4d %4d %4d %4d %4d %4d %4d %4d\n", 
					$Feeds{$InHost}->{InCons}, $InHost, $InSecs, $InIhave, 
					$InChk, $InRec, $InRej, $InErr, $InAdded if $Debug;
			}
		} elsif (/(... .. ..:..:..)\s+(.*)\s+newslink\[(\d+)\]:\s+(.*):(.*)\.S(.....)\s+final\s+secs=(\d+)\s+acc=(\d+)\s+dup=(\d+)\s+rej=(\d+)\s+tot=(\d+)/) {

			$OutDate = &GetYyMmDd ($1);

#			print STDERR "DATE  $GenDate  $OutDate\n" if $Debug;

			if ($OutDate >= $BegDate && $OutDate <= $EndDate) {
#				$NewslinkHost = $2;
#				$NewslinkPid = $3;
				$OutHost = $4;
				$OutBatch = $5;
				$OutBatchNum = $6;
				$OutSecs = $7;
				$OutAcc = $8;
				$OutDup = $9;
				$OutRej = $10;
				$OutTot = $11;

				$OutHost =~ s/\s//g;
				$BegTime = $OutDate if !$BegTime;
				$EndTime = $OutDate;

				$Feeds{$OutHost}->{OutCons} += 1;
				$Feeds{$OutHost}->{OutSecs} += $OutSecs;
				$Feeds{$OutHost}->{OutAcc} += $OutAcc;
				$Feeds{$OutHost}->{OutDup} += $OutDup;
				$Feeds{$OutHost}->{OutRej} += $OutRej;
				$Feeds{$OutHost}->{OutTot} += $OutTot;

				printf STDERR "OUT %-30s %4d %4d %4d %4d %4d %4d\n", 
					$Feeds{$OutHost}->{OutCons}, $OutHost, $OutSecs, 
					$OutAcc, $OutDup, $OutRej, $OutTot if $Debug;
			}
		}
	}
	close (FILE);

	$Num = 0;
	foreach $Host (sort keys %Feeds) {
		# InCons
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{InCons}, $Num;
		$OrderByInCons{$Key} = $Host;
		# InSecs
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{InSecs}, $Num;
		$OrderByInSecs{$Key} = $Host;
		# InIhave
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{InIhave}, $Num;
		$OrderByInIhave{$Key} = $Host;
		# InChk
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{InChk}, $Num;
		$OrderByInChk{$Key} = $Host;
		# InRec
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{InRec}, $Num;
		$OrderByInRec{$Key} = $Host;
		# InRej
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{InRej}, $Num;
		$OrderByInRej{$Key} = $Host;
		# InErr
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{InErr}, $Num;
		$OrderByInErr{$Key} = $Host;
		# InAdded
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{InAdded}, $Num;
		$OrderByInAdded{$Key} = $Host;

		# OutCons
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{OutCons}, $Num;
		$OrderByOutCons{$Key} = $Host;
		# OutSecs
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{OutSecs}, $Num;
		$OrderByOutSecs{$Key} = $Host;
		# OutAcc
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{OutAcc}, $Num;
		$OrderByOutAcc{$Key} = $Host;
		# OutDup
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{OutDup}, $Num;
		$OrderByOutDup{$Key} = $Host;
		# OutRej
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{OutRej}, $Num;
		$OrderByOutRej{$Key} = $Host;
		# OutTot
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{OutTot}, $Num;
		$OrderByOutTotal{$Key} = $Host;

		$Num++;
	}
}


sub ParseOutQueue
{
	my ($Key, $Host, $Num);

	print STDERR "Parsing output of $QueueCmd ...\n" if $Verbose;

	open (CMD, "$QueueCmd |") || die "Error: $QueueCmd - $!\n";

	while (<CMD>) 
	{
#		print if $Debug;

		chop;

		# news.ecrc.de  2699-2700  (  1/200 files   0% full)	02699 
		if (/^(.*)\s+(\d+)-(\d+)\s+\((.*)\/(.*)\s+files\s+(\d+)%\s+full\)/) {

			$Host = $1;
			$BatchBeg = $2;
			$BatchEnd = $3;
			$BatchNum = $4;
			$BatchMax = $5;
			$BatchFull = $6;

			$Feeds{$Host}->{BatchBeg} += $BatchBeg;
			$Feeds{$Host}->{BatchEnd} += $BatchEnd;
			$Feeds{$Host}->{BatchNum} += $BatchNum;
			$Feeds{$Host}->{BatchMax} += $BatchMax;
			$Feeds{$Host}->{BatchFull} += $BatchFull;

			printf STDERR "BATCH  %-30s %4d %4d %4d %4d %4d %4d %4d %4d\n", 
				$Feeds{$Host}->{BatchBeg}, $Feeds{$Host}->{BatchEnd},
				$Feeds{$Host}->{BatchNum}, $Feeds{$Host}->{BatchMax},
				$Feeds{$Host}->{BatchFull} if $Debug;
		}
	}
	close CMD;

	$Num = 0;
	foreach $Host (sort keys %Feeds) {
		$Key = sprintf "%10d.%3d", $Feeds{$Host}->{BatchFull}, $Num;
		$OrderByBatchFull{$Key} = $Host;
		$Num++;
	}
}


sub GenerateOutputFile
{
	if ($OutFile) {
		open (FILE, "> $OutFile") || warn "Warning: $OutFile - $!\n";
	} else {
		open (FILE, "> STDOUT") || warn "Warning: STDOUT - $!\n";
	}

	&PrintOutHeader;
 
#	&PrintChart (undef, "Incoming feeds by articles per half hour", undef);

	print FILE "<p>\n<a name=\"01\">\n<hr>\n<p>\n" if !$TextFormat;
	&PrintIncomingFeeds ("1. Summary of Incoming Feeds");

	&PrintSeparator ("02") if !$TextFormat;
	&PrintOutgoingFeeds ("2. Summary of Outgoing Feeds");

	&PrintSeparator ("03") if !$TextFormat;
	&PrintQueueStats ("3. Summary of Outgoing Queue");

	&PrintSeparator ("end") if !$TextFormat;

	&PrintOutFooter;

	close FILE;
}


sub CreateHtmlIndex
{
	my ($IndexFile) = "$HtmlDir/index.html";
	my ($Title1) = "DIABLO news relay statistics for '$DiabloHost'";
	my ($Title2) = "<a href=\"http://www.backplane.com/diablo/\">DIABLO</a>  news relay statistics for '$DiabloHost'";

	print STDERR "INDEX=[$IndexFile]\n" if $Debug;
	print STDERR "TITLE=[$Title]\n" if $Debug;

	open (DIR, "cd $HtmlDir; find . -name \"[0-9]*.html\" -print | sort -r |") || 
		die "Error: $HtmlDir - $!\n";

	open (FILE, ">$IndexFile") || die "Error: $IndexFile - $!\n";

	print FILE <<EOT
<html>
<head>
<title>$Title1</title>
</head>
<body>
<h2>$Title2</h2>
<hr>
<p>
<table border=2>
<th>Statistics for date
EOT
;

	while (<DIR>) {
		chop;
		s/^\.\///;
		if (/^(......)\.html/) {
			$Date = $1;
			$FancyDate = &GetFancyDate ($Date);
			print FILE "<tr>\n<td><a href=\"$Date.html\">$FancyDate</a>\n";
		}
	}
	print FILE <<EOT
</table>
<p>
<hr>
EOT
;
	&PrintOutFooter;

	close (FILE);
	close (DIR);
}

	
sub PrintChart 
{
	my ($hashref,$title,$AK) = @_;
	my ($Max) = 1;
	my ($Num) = 0;
	my (%TOD);
	my ($Date);
	my ($Width) = 8; # 10
	my (@ReqsPercent);
	my (%MonthName) = (
		'01','Jan','02','Feb','03','Mar','04','Apr','05','May','06','Jun',
		'07','Jul','08','Aug','09','Sep','10','Oct','11','Nov','12','Dec');

	$BegTime =~ /(..)(..)(..)/;
	$BegTime = "$1:$2:$3";
	$EndTime =~ /(..)(..)(..)/;
	$EndTime = "$1:$2:$3";
	$GenDate =~ /(....)(..)(..)/;
	$BegTime = $MonthName{$2}." ".int($3)." ".$BegTime." ".$1;
	$EndTime = $MonthName{$2}." ".int($3)." ".$EndTime." ".$1;

	print <<EOT
<table border=2 width=100%>
<caption><h2>Statistics from [$BegTime] to [$EndTime]</h2></caption>
<th>$title<th><th>Requests
<tr>
<td rowspan=12>
EOT
;
	foreach $i (0..47) {
		$Num = $ReqsPerHalfHour[$i];
       	$Max = $Num if ($Num > $Max);
	}
	$ReqPerPix = $Max / $MaxHeight;
#	print STDERR "IL: Max=[$Max] Height=[$MaxHeight] Req/Pix=[$ReqPerPix]\n";
	
	foreach $i (0..47) {
		$Reqs = $ReqsPerHalfHour[$i];
		$Height = int ($Reqs / $ReqPerPix);
		$Height = 1 if $Height < 1;

#		printf STDERR "Reqs=[%5d] Pix=[%5d]\n",  $Reqs, $Height;

		$Percent = ($Reqs / $Max) * 100;
		$Image = "$ImgDir/000-000.gif" if ($Percent == 0);
		$Image = "$ImgDir/000-010.gif" if ($Percent > 0);
		$Image = "$ImgDir/010-020.gif" if ($Percent > 10);
		$Image = "$ImgDir/020-030.gif" if ($Percent > 20);
		$Image = "$ImgDir/030-040.gif" if ($Percent > 30);
		$Image = "$ImgDir/040-050.gif" if ($Percent > 40);
		$Image = "$ImgDir/050-060.gif" if ($Percent > 50);
		$Image = "$ImgDir/060-070.gif" if ($Percent > 60);
		$Image = "$ImgDir/070-080.gif" if ($Percent > 70);
		$Image = "$ImgDir/080-090.gif" if ($Percent > 80);
		$Image = "$ImgDir/090-100.gif" if ($Percent > 90);

		print "<img src=\"$Image\" border=0 width=$Width height=$Height>";
	}

	foreach $i (0 .. 9) {
		$ReqsPercent[$i] = int(($Max / 10) * ($i+1));
	}
	
	$BarWidth = $Width * 48;
	
	print <<EOT
<br>
<img src="$ImgDir/ampm-bar.gif" width=$BarWidth height=22 border=0 hspace=0 vspace=0>
</td>
<tr>
<td><strong>Total<strong>
<td align="right">$TotalRequests
<tr>
<td><img src="$ImgDir/090-100.gif" width=10 height=10> &gt 90%
<td align="right">$ReqsPercent[9]
<tr>
<td><img src="$ImgDir/080-090.gif" width=10 height=10> &gt 80%
<td align="right">$ReqsPercent[8]
<tr>
<td><img src="$ImgDir/070-080.gif" width=10 height=10> &gt 70%
<td align="right">$ReqsPercent[7]
<tr>
<td><img src="$ImgDir/060-070.gif" width=10 height=10> &gt 60%
<td align="right">$ReqsPercent[6]
<tr>
<td><img src="$ImgDir/050-060.gif" width=10 height=10> &gt 50%
<td align="right">$ReqsPercent[5]
<tr>
<td><img src="$ImgDir/040-050.gif" width=10 height=10> &gt 40%
<td align="right">$ReqsPercent[4]
<tr>
<td><img src="$ImgDir/030-040.gif" width=10 height=10> &gt 30%
<td align="right">$ReqsPercent[3]
<tr>
<td><img src="$ImgDir/020-030.gif" width=10 height=10> &gt 20%
<td align="right">$ReqsPercent[2]
<tr>
<td><img src="$ImgDir/010-020.gif" width=10 height=10> &gt 10%
<td align="right">$ReqsPercent[1]
<tr>
<td><img src="$ImgDir/000-010.gif" width=10 height=10> &gt 0%
<td align="right">$ReqsPercent[0]
</table>
EOT
;
}

sub PrintSeparator
{
	my ($Num) = @_;

	if ($Num =~ /beg/i) {
		print FILE "<p>\n<hr>\n<p>\n";
	} elsif ($Num =~ /end/i) {
		print FILE "<a href=\"#00\">Goto Top</a>\n";
		print FILE "<p>\n<hr>\n";
	} else {
		print FILE "<a name=\"$Num\">\n";
		print FILE "<a href=\"#00\">Goto Top</a>\n";
		print FILE "<p>\n<hr>\n<p>\n";
	}
}


sub PrintIncomingFeeds
{
	my ($Title) = @_;
	my ($InCons, $InSecs, $InIhave, $InChk, $InErr, $InRej, $InRec, $InAdded);
	my ($Key);

	&PrintTableHeader ($Title);

	if ($TextFormat) {
		printf FILE "\n%-30s %8s %8s %8s %8s %8s %8s %8s %8s\n\n",
			"Incoming Feed", "Cons", "Secs", "IHave", "Check",
			"Err", "Rejected", "Received", "Total";
	} else {
		print FILE "<th>Incoming Feed<th>Cons<th>Secs<th>IHave<th>Check<th>Err<th>Rejected<th>Received<th>Total\n";
	}

	foreach $Key (reverse sort keys %OrderByInAdded) {
		$Host = $OrderByInAdded{$Key};

		if ($Feeds{$Host}->{InSecs} && $Feeds{$Host}->{InAdded}) {

			if ($TextFormat) {
				printf FILE "%-30s %8d %8d %8d %8d %8d %8d %8d %8d\n",
					$Host, $Feeds{$Host}->{InCons}, $Feeds{$Host}->{InSecs},
					$Feeds{$Host}->{InIhave}, $Feeds{$Host}->{InChk},
					$Feeds{$Host}->{InErr}, $Feeds{$Host}->{InRej},
					$Feeds{$Host}->{InRec}, $Feeds{$Host}->{InAdded};
			} else {
				printf FILE "<tr>\n%s%s %s%d %s%d %s%d %s%d %s%d %s%d %s%d %s%d\n",
					$TDL, $Host, 
					$TDR, $Feeds{$Host}->{InCons}, 
					$TDR, $Feeds{$Host}->{InSecs}, 
					$TDR, $Feeds{$Host}->{InIhave}, 
					$TDR, $Feeds{$Host}->{InChk}, 
					$TDR, $Feeds{$Host}->{InErr}, 
					$TDR, $Feeds{$Host}->{InRej},
					$TDR, $Feeds{$Host}->{InRec}, 
					$TDR, $Feeds{$Host}->{InAdded};

			}

			$InCons += $Feeds{$Host}->{InCons};
			$InSecs += $Feeds{$Host}->{InSecs};
			$InIhave += $Feeds{$Host}->{InIhave};
			$InChk += $Feeds{$Host}->{InChk};
			$InErr += $Feeds{$Host}->{InErr};
			$InRej += $Feeds{$Host}->{InRej};
			$InRec += $Feeds{$Host}->{InRec};
			$InAdded += $Feeds{$Host}->{InAdded};
		}
	}

	if ($TextFormat) {
		printf FILE "\n%s %8d %8d %8d %8d %8d %8d %8d %8d\n\n",
			"                              ",
			$InCons, $InSecs, $InIhave, $InChk,
			$InErr, $InRej, $InRec, $InAdded;
	} else {
		printf FILE "<tr>\n%s%s %s%d %s%d %s%d %s%d %s%d %s%d %s%d %s%d\n",
			$TDL, "",
			$TDR, $InCons,
			$TDR, $InSecs,
			$TDR, $InIhave,
			$TDR, $InChk,
			$TDR, $InErr,
			$TDR, $InRej,
			$TDR, $InRec,
			$TDR, $InAdded;
	}

	&PrintTableFooter;
}


sub PrintOutgoingFeeds
{
	my ($Title) = @_;
	my ($OutCons, $OutSecs, $OutDup, $OutRej, $OutAcc, $OutTot);
	my ($Key);

	&PrintTableHeader ($Title);

	if ($TextFormat) {
		printf FILE "\n%-30s %8s %8s %9s %8s %8s %8s\n\n",
			"Outgoing Feed", "Cons", "Secs", 
			"Duplicate", "Rejected", "Accepted", "Total";
	} else {
		print FILE "<th>Outgoing Feed<th>Cons<th>Secs<th>Duplicate<th>Rejected<th>Accepted<th>Total\n";
	}

	foreach $Key (reverse sort keys %OrderByOutTotal) {
		$Host = $OrderByOutTotal{$Key};
		if ($Feeds{$Host}->{OutSecs} && $Feeds{$Host}->{OutTot}) {

			if ($TextFormat) {
				printf FILE "%-30s %8d %8d %9d %8d %8d %8d\n",
					$Host,  $Feeds{$Host}->{OutCons}, $Feeds{$Host}->{OutSecs},
					$Feeds{$Host}->{OutDup}, $Feeds{$Host}->{OutRej}, 
					$Feeds{$Host}->{OutAcc}, $Feeds{$Host}->{OutTot};
			} else {
				printf FILE "<tr>\n%s%s %s%d %s%d %s%d %s%d %s%d %s%d\n",
					$TDL, $Host,
					$TDR, $Feeds{$Host}->{OutCons}, 
					$TDR, $Feeds{$Host}->{OutSecs},
					$TDR, $Feeds{$Host}->{OutDup}, 
					$TDR, $Feeds{$Host}->{OutRej}, 
					$TDR, $Feeds{$Host}->{OutAcc}, 
					$TDR, $Feeds{$Host}->{OutTot};
			}

			$OutCons += $Feeds{$Host}->{OutCons};
			$OutSecs += $Feeds{$Host}->{OutSecs};
			$OutDup += $Feeds{$Host}->{OutDup};
			$OutRej += $Feeds{$Host}->{OutRej};
			$OutAcc += $Feeds{$Host}->{OutAcc};
			$OutTot += $Feeds{$Host}->{OutTot};
		}
	}

	if ($TextFormat) {
		printf FILE "\n%s %8d %8d %9d %8d %8d %8d\n\n",
			"                              ",
			$OutCons, $OutSecs, $OutDup, $OutRej, $OutAcc, $OutTot;
	} else {
		printf FILE "<tr>\n%s%s %s%d %s%d %s%d %s%d %s%d %s%d\n",
			$TDL, "",
			$TDR, $OutCons,
			$TDR, $OutSecs,
			$TDR, $OutDup,
			$TDR, $OutRej,
			$TDR, $OutAcc,
			$TDR, $OutTot;
	}

	&PrintTableFooter;
}


sub PrintQueueStats
{
	my ($Title) = @_;
	my ($OutCons, $OutSecs, $OutDup, $OutRej, $OutAcc, $OutTot);
	my ($Key, $Color, $TDL, $TDR);

	if (! $TextFormat) {
		print FILE <<EOT
<table border=2 width=100%>
<td align="center" bgcolor=$QueColor1><strong>&gt;= $QuePercent1% Full</strong></td>
<td align="center" bgcolor=$QueColor2><strong>&gt;= $QuePercent2% Full</strong></td>
<td align="center" bgcolor=$QueColor3><strong>&gt;= $QuePercent3% Full</strong></td>
<td align="center" bgcolor=$QueColor4><strong>&gt;= $QuePercent4% Full</strong></td>
<td align="center" bgcolor=$QueColor5><strong>&gt;= $QuePercent5% Full</strong></td>
</table>
<p>
EOT
;
	}

	&PrintTableHeader ($Title);

	if ($TextFormat) {
		printf FILE "\n%-30s %9s %9s %9s %9s %9s\n\n",
			"Outgoing Feed", "Batch Beg", "Batch End", 
			"Batch Num", "Batch Max", "%Full";
	} else {
		print FILE "<th>Outgoing Feed<th>Batch Beg<th>Batch End<th>Batch Num<th>Batch Max<th>%Full\n";
	}

	foreach $Key (reverse sort keys %OrderByBatchFull) {
		$Host = $OrderByBatchFull{$Key};

		if ($Feeds{$Host}->{BatchMax}) {

			if ($TextFormat) {
				printf FILE "%-30s %9d %9d %9d %9d %9d\n",
					$Host,
					$Feeds{$Host}->{BatchBeg}, 
					$Feeds{$Host}->{BatchEnd},
					$Feeds{$Host}->{BatchNum},
					$Feeds{$Host}->{BatchMax},
					$Feeds{$Host}->{BatchFull};
			} else {
				if ($Feeds{$Host}->{BatchFull} >= $QuePercent5) {
					$Color = "bgcolor=$QueColor5";
				} elsif ($Feeds{$Host}->{BatchFull} >= $QuePercent4) {
					$Color = "bgcolor=$QueColor4";
				} elsif ($Feeds{$Host}->{BatchFull} >= $QuePercent3) {
					$Color = "bgcolor=$QueColor3";
				} elsif ($Feeds{$Host}->{BatchFull} >= $QuePercent2) {
					$Color = "bgcolor=$QueColor2";
				} elsif ($Feeds{$Host}->{BatchFull} >= $QuePercent1) {
					$Color = "bgcolor=$QueColor1";
				} else {
					$Color = "";
				}
				$TDL = "<td $Color>";
				$TDR = "<td $Color align=\"right\">";

				printf FILE "<tr>\n%s%s %s%d %s%d %s%d %s%d %s%d\n",
					$TDL, $Host,
					$TDR, $Feeds{$Host}->{BatchBeg}, 
					$TDR, $Feeds{$Host}->{BatchEnd},
					$TDR, $Feeds{$Host}->{BatchNum},
					$TDR, $Feeds{$Host}->{BatchMax},
					$TDR, $Feeds{$Host}->{BatchFull};
			}
		}
	}

	&PrintTableFooter;
}


sub PrintTableHeader
{
	my ($Title) = @_;

	if ($TextFormat) {
		print FILE "$Title\n";
	} else {
		print FILE <<EOT
<table border=2 width=100%>
<caption><h2>$Title</h2></caption>
EOT
;
	}
}


sub PrintTableFooter
{
	print FILE "</table>\n" if !$TextFormat;
}


sub ParseReqsPerHalfHour
{
	my ($Time) = @_;
	my ($Idx) = 0;
	my ($Num);

	foreach $Num (0..47) {
		if ($Time >= $HalfHour[$Idx++] && $Time <= $HalfHour[$Idx++]) {
			$ReqsPerHalfHour[$Num] = $ReqsPerHalfHour[$Num] + 1;
#			print STDERR "[$Time]  $HalfHour[$Idx-2] - $HalfHour[$Idx-1]  $ReqsPerHalfHour[$Num]\n";
			return;
		}
	}
}

sub InitReqsPerHalfHour
{
	@HalfHour = (
		'000000', '002959',
		'003000', '005959',
		'010000', '012959',
		'013000', '015959',
		'020000', '022959',
		'023000', '025959',
		'030000', '032959',
		'033000', '035959',
		'040000', '042959',
		'043000', '045959',
		'050000', '052959',
		'053000', '055959',
		'060000', '062959',
		'063000', '065959',
		'070000', '072959',
		'073000', '075959',
		'080000', '082959',
		'083000', '085959',
		'090000', '092959',
		'093000', '095959',
		'100000', '102959',
		'103000', '105959',
		'110000', '112959',
		'113000', '115959',
		'120000', '122959',
		'123000', '125959',
		'130000', '132959',
		'133000', '135959',
		'140000', '142959',
		'143000', '145959',
		'150000', '152959',
		'153000', '155959',
		'160000', '162959',
		'163000', '165959',
		'170000', '172959',
		'173000', '175959',
		'180000', '182959',
		'183000', '185959',
		'190000', '192959',
		'193000', '195959',
		'200000', '202959',
		'203000', '205959',
		'210000', '212959',
		'213000', '215959',
		'220000', '222959',
		'223000', '225959',
		'230000', '232959',
		'233000', '235959');
}


sub PrintOutHeader
{
	my ($TimePeriod) = $BegDate;
	my ($Title);

	if ($BegDate ne $EndDate) {
		$TimePeriod = "$BegDate - $EndDate";
	}

	$Title = "DIABLO statistics for '$DiabloHost' for $TimePeriod";

	if ($TextFormat) {
		print FILE "$Title\n\n\n";
	} else {
		print FILE <<EOT
<html>
<head>
<title>$Title</title>
</head>
<body>
<a name="00">
<h2>$Title</h2>
<hr>
<p>
<ol>
<li><a href="#01">Summary of Incoming Feeds</a>
<li><a href="#02">Summary of Outgoing Feeds</a>
<li><a href="#03">Summary of Outgoing Queue</a>
</ol>
EOT
;
	}
}


sub PrintOutFooter
{
	if ($TextFormat) {
		print FILE "\nGenerated on $GenDate $GenTime by $ScriptName $Version. Copyright (C) 1997 Iain Lea (iain\@ecrc.de)\n";
	} else {
		print FILE <<EOT
Generated on $GenDate $GenTime by 
<a href="$Url">$ScriptName $Version</a>. 
Copyright &copy; 1997 
<a href="mailto:iain\@ecrc.de">Iain Lea</a>. 
</body>
</html>	
EOT
;
	}
}


sub GetYyMmDd
{
	my ($Date) = @_;

	# Apr 10 11:36:54
	if ($Date =~ /^(...) (..)/) {
		return sprintf "%02d%02d%02d", $GenYear, $MonthNumByName{$1}, $2;
	} else {
		return "";
	}
}


sub GetFancyDate
{
	my ($Date) = @_;

	if (/^(..)(..)(..)/) {
		$Date = sprintf "%s %s %s", $3, $MonthNameByNum{$2}, $1;
	}

	return $Date;
}

